{
****************************************************************
*  Name    : S1D10605.BAS                                      *
*  Author  : David John Barker                                 *
*  Notice  : Copyright (c) 2006 Mecanique                      *
*          : All Rights Reserved                               *
*  Date    : 02/05/2006                                        *
*  Version : 1.3                                               *
*  Notes   : 1.3 add utils.bas for SetAllDigital               *   
*          : 1.2 add #option GLCD_SETALLDIGITAL                *   
*  Notes   : Tested with Batron COG-BTD12864-04                *
****************************************************************
}
module S1D10605

// import the graphics module...
#define GLCD_PIXEL_01
#define GLCD_COLOR_01
#define GLCD_XY_08
include "Graphics.bas"
include "system.bas"  
include "utils.bas"

// default module options - user options can override these values...
#option GLCD_DATA = PORTD        // data port
#option GLCD_DC = PORTE.2        // data or command
#option GLCD_EN = PORTE.0        // EN pin
#option GLCD_RW = PORTC.0        // RW pin
#option GLCD_CS1 = PORTE.1       // chip select
#option GLCD_RST = PORTC.1       // reset
#option GLCD_ASPECT_RATIO = 75   // aspect ratio, smaller number will squeeze y for GLCD circles and box
#option GLCD_INIT_DELAY = 100    // initialisation delay (ms)

// by default, the module calls SetAllDigital in Initialize()
// set option false to exclude the call
#if IsOption(GLCD_SETALLDIGITAL) and not (GLCD_SETALLDIGITAL in (true, false))
   #error GLCD_SETALLDIGITAL, "Invalid option"
#endif
#option GLCD_SETALLDIGITAL = true

// validate data port...
#if IsOption(GLCD_DATA) 
   #if not IsValidPort(GLCD_DATA)
      #error GLCD_DATA, "Invalid option. DATA must be a valid port name."
   #endif
   #ifdef GLCD_DATA@ then
      #error GLCD_DATA@, "Invalid option. DATA port cannot be a single bit value."
   #endif
#endif

// validate RS pin...
#if IsOption(GLCD_DC) and not IsValidPortPin(GLCD_DC) 
   #error GLCD_DC, "Invalid option. RS must be a valid port pin."
#endif

// validate EN pin...
#if IsOption(GLCD_EN) and not IsValidPortPin(GLCD_EN) 
   #error GLCD_EN, "Invalid option. EN must be a valid port pin."
#endif

// validate RW pin...
#if IsOption(GLCD_RW) and not IsValidPortPin(GLCD_RW) 
   #error GLCD_RW, "Invalid option. RW must be a valid port pin."
#endif

// validate CS1 pin...
#if IsOption(GLCD_CS1) and not IsValidPortPin(GLCD_CS1) 
   #error GLCD_CS1, "Invalid option. CS1 must be a valid port pin."
#endif

// validate CS2 pin...
#if IsOption(GLCD_RST) and not IsValidPortPin(GLCD_RST) 
   #error GLCD_RST, "Invalid option. RST must be a valid port pin."
#endif

// validate initialisation delay...
#if IsOption(GLCD_INIT_DELAY)
   #if not (GLCD_INIT_DELAY in (0 to 1000))
      #error GLCD_INIT_DELAY, "Invalid option. GLCD initialize delay must be between 0 and 1000 (ms)."
   #endif
#endif 

// now create Data TRIS...
#option _GLCD_DATA_TRIS = GetTRIS(GLCD_DATA)

// GLCD width and height...
public const
   GLCDWidth = 128,
   GLCDHeight = 64

// x, y position...
public dim
   Pos as TPosition

// SD10605 commands...   
const
   cmdOn = $AF,
   cmdOff = $AE,
   cmdStartLine = $40,
   cmdPage = $B0,
   cmdColumnMS = $10,
   cmdColumnLS = $0F,   
   cmdADCNormal = $A0,
   cmdADCReverse = $A1,
   cmdDisplayNormal = $A6,
   cmdDisplayReverse = $A7,
   cmdDisplayAllOff = $A4,
   cmdDisplayAllOn = $A5,
   cmdBias19 = $A2,
   cmdBias17 = $A3,
   cmdModify = $E0,
   cmdEnd = $EE,
   cmdReset = $E2,
   cmdScanTopBottom = $C0,
   cmdScanBottomTop = $C8,
   cmdPowerControl = $28,
   cmdV5Voltage = $20,
   cmdContrast = $81, 
   cmdStaticOn = $AD,
   cmdStaticOff = $AC,
   cmdNop = $E3, 
   GLCDDelay = GLCD_INIT_DELAY     

// port and pin settings, these are brought into
// the program by using the above options...
dim        
   DATA as GLCD_DATA,           // data in (PORT) 
   TRISData as _GLCD_DATA_TRIS, // data TRIS
   DC as GLCD_DC.GLCD_DC@,      // DC pin (data or command)
   EN as GLCD_EN.GLCD_EN@,      // EN pin
   RW as GLCD_RW.GLCD_RW@,      // RW pin (read or write)
   CS1 as GLCD_CS1.GLCD_CS1@,   // chip select
   RST as GLCD_RST.GLCD_RST@    // reset
 
{
****************************************************************************
* Name    : StrobeEN (PRIVATE)                                             *
* Purpose : Strobe the GLCD EN pin                                         *
****************************************************************************
}    
inline sub StrobeEN()
   CS1 = 0
   EN = 1
   delayus(1)
   EN = 0 
   CS1 = 1
end sub
{
****************************************************************************
* Name    : GLCDData (PRIVATE)                                             *
* Purpose : Switch to GLCD data mode                                       *
****************************************************************************
}    
inline sub GLCDData()
   DC = 1
end sub
{
****************************************************************************
* Name    : GLCDInst (PRIVATE)                                             *
* Purpose : Switch to GLCD instruction mode                                *
****************************************************************************
}    
inline sub GLCDInst()
   DC = 0   
end sub
{
****************************************************************************
* Name    : GLCDRead (PRIVATE)                                             *
* Purpose : Set GLCD to read mode                                          *
****************************************************************************
}    
inline sub GLCDRead()
   RW = 1
end sub
{
****************************************************************************
* Name    : GLCDWrite (PRIVATE)                                            *
* Purpose : Set GLCD to write mode                                         *
****************************************************************************
}    
inline sub GLCDWrite()
   RW = 0   
end sub
{
****************************************************************************
* Name    : WaitForIdle (PRIVATE)                                          *
* Purpose : Block further GLCD access until signalled ready.               *
****************************************************************************
}     
sub WaitForIdle()
   dim Timeout as byte
   Timeout = $FF 
   TRISData = $FF     // set data bus to input
   GLCDInst           // instruction data
   GLCDRead           // read mode
   repeat
      ClrWDT
      dec(Timeout)
      StrobeEN 
   until (DATA.7 = 0) or (Timeout = 0)
end sub 
{
****************************************************************************
* Name    : SetData (PRIVATE)                                              *
* Purpose : Write a data byte to GLCD                                      *
****************************************************************************
}     
sub SetData(pData as byte) 
   WaitForIdle      // block until not busy
   TRISData = $00   // set data bus to output
   GLCDData         // access display RAM data
   GLCDWrite        // write mode
   DATA = pData     // write to the bus
   StrobeEN         // write to GLCD
end sub
{
****************************************************************************
* Name    : GetData (PRIVATE)                                              *
* Purpose : Read byte from GLCD                                            *
****************************************************************************
}     
function GetData() as byte
   WaitForIdle      // block until not busy
   TRISData = $FF   // set data bus to input
   GLCDData         // access display RAM data
   GLCDRead         // read mode
   StrobeEN         // latch data
   Result = DATA    // get the data
end function  
{
****************************************************************************
* Name    : Command (PRIVATE)                                              *
* Purpose : Write a command byte to GLCD                                   *
****************************************************************************
}     
sub Command(pCommand as byte) 
   WaitForIdle          // block until not busy
   TRISData = $00       // set data bus to output
   GLCDInst             // instruction mode
   GLCDWrite            // write mode
   DATA = pCommand      // write to the bus
   StrobeEN             // write to GLCD
end sub
{
****************************************************************************
* Name    : CommandWithData (PRIVATE)                                      *
* Purpose : Write a command plus extra data byte to GLCD                   *
****************************************************************************
}     
sub CommandWithData(pCommand,pData as byte) 
   WaitForIdle          // block until not busy
   TRISData = $00       // set data bus to output
   GLCDInst             // instruction mode
   GLCDWrite            // write mode
   DATA = pCommand      // write to the bus
   StrobeEN             // write to GLCD
   DATA = pData
   StrobeEN
end sub
{
****************************************************************************
* Name    : SetPosition (PRIVATE)                                          *
* Purpose : Set GLCD x and y positions                                     *
****************************************************************************
}     
sub SetPosition()     
   Command(cmdPage or Pos.y)           // set y position
   Command($10 or (Pos.x + 4 >> 4))    // set x, upper nibble
   Command($0F and Pos.x + 4)          // set x, lower nibble
end sub
{
****************************************************************************
* Name    : WriteByte                                                      *
* Purpose : Write a byte at x, page                                        *
****************************************************************************
} 
public sub WriteByte(pValue as byte)
   SetPosition
   SetData(pValue)
end sub 
{
****************************************************************************
* Name    : ReadByte                                                       *
* Purpose : Read a byte at x, page                                         *
****************************************************************************
} 
public function ReadByte() as byte
   SetPosition
   GetData
   Result = GetData
end function
{
****************************************************************************
* Name    : SetPixel                                                       *
* Purpose : Set pixel at pixel location x,y                                *
****************************************************************************
} 
public sub SetPixel(pX, pY as byte)
   dim YBit as byte
   dim Pixel as byte
   
   if (pX < GLCDWidth) and (pY < GLCDHeight) then

      YBit = 1 << (pY mod 8)
      Pos.y = pY / 8
      Pos.x = pX
   
      SetPosition
      GetData
      Pixel = GetData  

      // pen is white...
      if Pen.Color = 0 then
         if Pen.Mode = pmCopy then
            yBit = not yBit
            Pixel = Pixel and YBit
         endif
   
      // pen is black...
      else
      
         // pen copy or merge...
         if Pen.Mode <> pmXOR then
            Pixel = Pixel or yBit
      
         // pen XOR
         else
            if (Pixel and YBit) = 0 then
               Pixel = Pixel or yBit
            else
               yBit = not yBit
               Pixel = Pixel and YBit
            endif      
         endif
      endif
      SetPosition
      SetData(Pixel)
   endif
end sub
{
****************************************************************************
* Name    : GetPixel                                                       *
* Purpose : Return pixel colour at pixel position x, y                     *
****************************************************************************
}
public function GetPixel(pX, pY as byte) as TColor
   dim Pixel as byte
   Pos.y = pY / 8
   Pos.x = pX
   SetPosition
   GetData
   Pixel = GetData >> (pY mod 8) 
   Result = Pixel.0  
end function
{
****************************************************************************
* Name    : Cls                                                            *
* Purpose : Clear the GLCD screen                                          *
****************************************************************************
} 
public sub Cls()
   dim x, y as byte
   y = 0
   repeat   
      Command(cmdPage or y)
      x = 4
      repeat        
         Command(cmdColumnMS or (x >> 4)) 
         Command(cmdColumnLS and x) 
         SetData($00)
         inc(x)
      until x > 131
      inc(y)
   until y > 7  
end sub 
{
****************************************************************************
* Name    :  SetContrast                                                   *
* Purpose :  Set the GLCD contrast                                         *
****************************************************************************
}    
public sub SetContrast(pValue as byte)
   CommandWithData(cmdContrast,pValue)
end sub
{
****************************************************************************
* Name    : Sleep                                                          *
* Purpose : Put the GLCD to sleep or wake it up                            *
****************************************************************************
}    
public sub Sleep(pOn as boolean = true)
   if pOn then
      Command(cmdOff)
      Command(cmdDisplayAllOn)
      Command(cmdStaticOff)
   else
      Command(cmdDisplayAllOff)
      Command(cmdOn)
   endif   
end sub
{
****************************************************************************
* Name    : StandBy                                                        *
* Purpose : Put the GLCD into standby mode or wake it up                   *
****************************************************************************
}    
public sub StandBy(pOn as boolean = true)
   if pOn then
      Command(cmdOff)
      Command(cmdDisplayAllOn)
      CommandWithData(cmdStaticOn, 3)
   else
      Command(cmdStaticOff)
      Command(cmdDisplayAllOff)
      Command(cmdOn)
   endif
end sub
{
****************************************************************************
* Name    : Initialize                                                     *
* Purpose : Configure the GLCD before use                                  *
****************************************************************************
} 
sub Initialize()
  #if (GLCD_SETALLDIGITAL)
   SetAllDigital()
  #endif

   Pos.x = 0
   Pos.y = 0  
   delayms(GLCDDelay) // start up delay, allow GLCD to settle
   output(RW)         // read/write is low
   low(EN)            // enable is low
   output(DC)         // data or instruction is low (command mode)
   output(CS1)        // screen 
   high(RST)          // reset
      
   Command(cmdReset)                // software reset
   Command(cmdBias17)               // bias
   Command(cmdADCNormal)            // ADC output = normal
   Command(cmdDisplayAllOff)        // display all point normal
   Command(cmdV5Voltage or 4)       // internal resistor ratio (0..7)
   Command(cmdPowerControl or $07)  // internal power supply mode

   SetContrast(30)                  // LCD contrast
   Command(cmdStaticOff)            // static indicator is off
   Command(cmdOn)                   // display is on
   Command(cmdScanBottomTop)        // scan top to bottom
   Command(cmdDisplayNormal)        // display normal
   Command(cmdStartLine)            // display start line (or value for line number) 
end sub 

Initialize()

end module
